#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <err.h>
#include <X11/Xlib.h>

#include "timeutil.h"

char *argv0;

void
usage()
{
	errx(1, "usage: %s [-p prefix] [-P postfix] [-i interval] [-x]", argv0);
}

void
print_loop(unsigned int interval, char *prefix, char *postfix)
{
	char buf[LINE_MAX];
	time_t t_start = time(NULL);

	while (1) {
		format_time(buf, sizeof(buf), time(NULL) - t_start, prefix, postfix);
		printf("\r%s", buf);
		fflush(stdout);
		sleep(interval);
	}
}

void
xroot_loop(unsigned int interval, char *prefix, char *postfix)
{
	Display *dpy;
	char buf[LINE_MAX];
	time_t t_start = time(NULL);

	dpy = XOpenDisplay(NULL);
	if (dpy == NULL)
		errx(1, "cannot open display");
	while (1) {
		format_time(buf, sizeof(buf), time(NULL) - t_start, prefix, postfix);
		XStoreName(dpy, DefaultRootWindow(dpy), buf);
		XSync(dpy, False);
		sleep(interval);
	}
}

int
main(int argc, char *argv[])
{
	int ch, xflag = 0;
	unsigned int interval = 1;
	char prefix[LINE_MAX] = "", postfix[LINE_MAX] = "";
	const char *errstr;

	argv0 = *argv;

	while ((ch = getopt(argc, argv, "p:P:i:x")) != -1) {
		switch (ch) {
		case 'p':
			strlcpy(prefix, optarg, sizeof(prefix));
			break;
		case 'P':
			strlcpy(postfix, optarg, sizeof(postfix));
			break;
		case 'i':
			interval = strtonum(optarg, 1, UINT_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "interval is %s: %s", errstr, optarg);
			break;
		case 'x':
			xflag = 1;
			break;
		default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;
	if (argc > 0)
		usage();

	if (xflag)
		xroot_loop(interval, prefix, postfix);
	else
		print_loop(interval, prefix, postfix);

	return 0;
}
