#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <err.h>
#include <X11/Xlib.h>

#include "timeutil.h"

char *argv0;

void
usage(void)
{
	errx(1, "usage: %s [-p prefix] [-P postfix] [-i interval] [-x] HH:MM:SS", argv0);
}

void
print_loop(unsigned int interval, char *prefix, char *postfix, time_t duration)
{
	char buf[LINE_MAX];
	time_t t_start = time(NULL);
	time_t t_elapsed = 0;

	while ((t_elapsed = time(NULL) - t_start) < duration) {
		format_time(buf, sizeof(buf), duration - t_elapsed, prefix, postfix);
		printf("\r%s ", buf);
		fflush(stdout);
		sleep(interval);
	}
	printf("\a\n");
}

void
xroot_loop(unsigned int interval, char *prefix, char *postfix, time_t duration)
{
	Display *dpy;
	char buf[LINE_MAX];
	time_t t_start = time(NULL);
	time_t t_elapsed = 0;

	dpy = XOpenDisplay(NULL);
	if (dpy == NULL)
		errx(1, "cannot open display");
	while ((t_elapsed = time(NULL) - t_start) < duration) {
		format_time(buf, sizeof(buf), duration - t_elapsed, prefix, postfix);
		XStoreName(dpy, DefaultRootWindow(dpy), buf);
		XSync(dpy, False);
		sleep(interval);
	}
	snprintf(buf, sizeof(buf), "%sEND%s", prefix, postfix);
	XStoreName(dpy, DefaultRootWindow(dpy), buf);
	XSync(dpy, False);
}

int
main(int argc, char *argv[])
{
	int h, m, s;
	time_t duration;
	int ch, xflag = 0;
	unsigned int interval = 1;
	char prefix[LINE_MAX] = "", postfix[LINE_MAX] = "";
	const char *errstr;

	argv0 = *argv;

	while ((ch = getopt(argc, argv, "p:P:i:x")) != -1) {
		switch (ch) {
		case 'p':
			strlcpy(prefix, optarg, sizeof(prefix));
			break;
		case 'P':
			strlcpy(postfix, optarg, sizeof(postfix));
			break;
		case 'i':
			interval = strtonum(optarg, 1, UINT_MAX, &errstr);
			if (errstr != NULL)
				errx(1, "interval is %s: %s", errstr, optarg);
			break;
		case 'x':
			xflag = 1;
			break;
		default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;
	if (argc != 1)
		usage();

	if (sscanf(argv[0], "%d:%d:%d", &h, &m, &s) != 3)
		errx(2, "could not parse time in HH:MM:SS format");
	duration = h * 3600 + m * 60 + s;

	if (xflag)
		xroot_loop(interval, prefix, postfix, duration);
	else
		print_loop(interval, prefix, postfix, duration);

	return 0;
}
